@@ -1,4 +1,4 @@
-# Copyright (c) 2014-2020 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
+# Copyright (c) 2014-2021 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
@@ -10,20 +10,24 @@
 # You should have received a copy of the GNU General Public License
 # along with this program. If not, see <http://www.gnu.org/licenses/>.
 
-from gi.repository import Gtk
+from gi.repository import Gtk, GLib
 
 from random import shuffle
 
-from lollypop.utils import get_human_duration, tracks_to_albums, popup_widget
-from lollypop.define import App, ArtSize, ViewType, Size
+from lollypop.utils import get_human_duration, popup_widget
+from lollypop.utils_album import tracks_to_albums
+from lollypop.define import App, ArtSize, ViewType
+from lollypop.objects_track import Track
 from lollypop.widgets_banner import BannerWidget
+from lollypop.helper_signals import SignalsHelper, signals_map
 
 
-class PlaylistBannerWidget(BannerWidget):
+class PlaylistBannerWidget(BannerWidget, SignalsHelper):
     """
         Banner for playlist
     """
 
+    @signals_map
     def __init__(self, playlist_id, view):
         """
             Init banner
@@ -31,6 +35,7 @@ class PlaylistBannerWidget(BannerWidget):
             @param view as AlbumsListView
         """
         BannerWidget.__init__(self, view.args["view_type"] | ViewType.OVERLAY)
+        self.__duration_task = False
         self.__playlist_id = playlist_id
         self.__view = view
         builder = Gtk.Builder()
@@ -47,19 +52,24 @@ class PlaylistBannerWidget(BannerWidget):
         self._overlay.set_overlay_pass_through(widget, True)
         self.__title_label.set_label(App().playlists.get_name(playlist_id))
         builder.connect_signals(self)
-        # In DB duration calculation
-        if playlist_id > 0 and\
-                not App().playlists.get_smart(playlist_id):
-            duration = App().playlists.get_duration(playlist_id)
-            self.__duration_label.set_text(get_human_duration(duration))
-
-    def update_for_width(self, width):
-        """
-            Update banner internals for width, call this before showing banner
-            @param width as int
-        """
-        BannerWidget.update_for_width(self, width)
+        self.connect("destroy", self.__on_destroy)
         self.__set_internal_size()
+        return [
+            (view, "initialized", "_on_view_updated"),
+            (App().window.container.widget, "notify::folded",
+             "_on_container_folded"),
+            (App().player, "duration-changed", "_on_view_updated"),
+            (App().playlists, "playlist-track-added", "_on_playlist_changed"),
+            (App().playlists, "playlist-track-removed",
+             "_on_playlist_changed"),
+        ]
+
+    def rename(self, name):
+        """
+            Rename playlist
+            @param name as str
+        """
+        self.__title_label.set_label(name)
 
     @property
     def spinner(self):
@@ -80,13 +90,24 @@ class PlaylistBannerWidget(BannerWidget):
 #######################
 # PROTECTED           #
 #######################
-    def _handle_width_allocate(self, allocation):
+    def _on_container_folded(self, leaflet, folded):
         """
-            Update artwork
-            @param allocation as Gtk.Allocation
+            Handle libhandy folded status
+            @param leaflet as Handy.Leaflet
+            @param folded as Gparam
         """
-        if BannerWidget._handle_width_allocate(self, allocation):
-            self.__set_internal_size()
+        self.__set_internal_size()
+
+    def _on_view_updated(self, *ignore):
+        """
+            Update duration
+        """
+        if self.__view.children:
+            self.__duration_task = True
+            App().task_helper.run(self.__calculate_duration)
+        else:
+            self.__duration_task = False
+            self.__duration_label.set_text(get_human_duration(0))
 
     def _on_play_button_clicked(self, button):
         """
@@ -106,14 +127,15 @@ class PlaylistBannerWidget(BannerWidget):
             Play playlist shuffled
             @param button as Gtk.Button
         """
-        tracks = []
+        track_ids = []
         for album_row in self.__view.children:
             for track in album_row.album.tracks:
-                tracks.append(track)
-        if tracks:
-            shuffle(tracks)
-            albums = tracks_to_albums(tracks)
-            App().player.play_track_for_albums(tracks[0], albums)
+                track_ids.append(track.id)
+        if track_ids:
+            shuffle(track_ids)
+            albums = tracks_to_albums(
+                [Track(track_id) for track_id in track_ids])
+            App().player.play_track_for_albums(albums[0].tracks[0], albums)
 
     def _on_menu_button_clicked(self, button):
         """
@@ -122,21 +144,45 @@ class PlaylistBannerWidget(BannerWidget):
         """
         from lollypop.widgets_menu import MenuBuilder
         from lollypop.menu_playlist import PlaylistMenu, PlaylistMenuExt
-        menu = PlaylistMenu(self.__playlist_id, App().window.is_adaptive)
+        menu = PlaylistMenu(self.__playlist_id,
+                            self.view_type,
+                            App().window.folded)
         menu_widget = MenuBuilder(menu)
         if self.__playlist_id >= 0:
             menu_widget = MenuBuilder(menu)
             menu_ext = PlaylistMenuExt(self.__playlist_id)
             menu_ext.show()
-            menu_widget.append_widget(menu_ext)
+            menu_widget.add_widget(menu_ext)
         else:
             menu_widget = MenuBuilder(menu)
         menu_widget.show()
-        popup_widget(menu_widget, button)
+        popup_widget(menu_widget, button, None, None, button)
+
+    def _on_playlist_changed(self, playlists, playlist_id, uri):
+        """
+            Update duration
+            @param playlists as Playlists
+            @param playlist_id as int
+            @param uri as str
+        """
+        if self.__playlist_id == playlist_id:
+            self.__calculate_duration()
 
 #######################
 # PRIVATE             #
 #######################
+    def __calculate_duration(self):
+        """
+            Calculate playback duration
+        """
+        duration = 0
+        for child in self.__view.children:
+            if not self.__duration_task:
+                return
+            duration += child.album.duration
+        GLib.idle_add(self.__duration_label.set_text,
+                      get_human_duration(duration))
+
     def __set_internal_size(self):
         """
             Update font size
@@ -147,8 +193,22 @@ class PlaylistBannerWidget(BannerWidget):
             title_context.remove_class(c)
         for c in duration_context.list_classes():
             duration_context.remove_class(c)
-        if self.width <= Size.MEDIUM:
+        if App().window.folded:
             title_context.add_class("text-large")
         else:
             title_context.add_class("text-x-large")
             duration_context.add_class("text-large")
+
+    def __on_get_duration(self, duration):
+        """
+            Update label
+            @param duration as int
+        """
+        self.__duration_label.set_text(get_human_duration(duration))
+
+    def __on_destroy(self, widget):
+        """
+            Remove ref cycle
+            @param widget as Gtk.Widget
+        """
+        self.__view = None
